using System;
using System.Xml;
using System.Linq;
using System.Collections.Generic;

namespace Lab5
{
    public class Biblioteca
    {
        private readonly Dictionary<int, Libro> _libri = new Dictionary<int, Libro>();
        private readonly Dictionary<int, Persona> _persone = new Dictionary<int, Persona>();
        private readonly Dictionary<int, Prestito> _prestiti = new Dictionary<int, Prestito>();
        private int _nextIdLibro = 1;
        private int _nextIdPersona = 1;
        private bool _isDirty = false;

        public Biblioteca()
        { }

        public Biblioteca(XmlReader reader)
        {
            while (reader.Read())
            {
                if (reader.NodeType != XmlNodeType.Element)
                    continue;
                switch (reader.Name)
                {
                    case "Biblioteca":
                        {
                            _nextIdLibro = XmlConvert.ToInt32(reader["nextIdLibro"]);
                            _nextIdPersona = XmlConvert.ToInt32(reader["nextIdPersona"]);
                            break;
                        }
                    case "Libro":
                        {
                            Libro libro = new Libro(this, reader);
                            _libri.Add(libro.Id, libro);
                            break;
                        }
                    case "Persona":
                        {
                            Persona persona = new Persona(this, reader);
                            _persone.Add(persona.Id, persona);
                            break;
                        }
                    case "Prestito":
                        {
                            Prestito prestito = new Prestito(this, reader);
                            _prestiti.Add(prestito.IdLibro, prestito);
                            break;
                        }
                }
            }
        }

        public void Salva(XmlWriter writer)
        {
            writer.WriteStartDocument();
            writer.WriteStartElement("Biblioteca");
            writer.WriteAttributeString("nextIdLibro", XmlConvert.ToString(_nextIdLibro));
            writer.WriteAttributeString("nextIdPersona", XmlConvert.ToString(_nextIdPersona));

            writer.WriteStartElement("Libri");
            foreach (Libro libro in _libri.Values)
            {
                libro.Salva(writer);
            }
            writer.WriteEndElement();

            writer.WriteStartElement("Persone");
            foreach (Persona persona in _persone.Values)
            {
                persona.Salva(writer);
            }
            writer.WriteEndElement();

            writer.WriteStartElement("Prestiti");
            foreach (Prestito prestito in Prestiti.Values)
            {
                prestito.Salva(writer);
            }
            writer.WriteEndElement();

            writer.WriteEndElement();
            writer.WriteEndDocument();

            _isDirty = false;
        }

        public bool IsDirty
        {
            get { return _isDirty; }
        }

        //  --- Gestione libri ---

        public int GetNextIdLibro()
        {
            return _nextIdLibro++;
        }

        public Libro GetLibro(int id)
        {
            return _libri[id];
        }

        public IEnumerable<Libro> GetLibri()
        {
            return _libri.Values;
        }

        public bool Contains(Libro libro)
        {
            if (libro == null)
                throw new ArgumentNullException("libro");
            return _libri.ContainsKey(libro.Id);
        }

        public bool Aggiungi(Libro libro)
        {
            if (Contains(libro))
                return false;
            _libri.Add(libro.Id, libro);
            _isDirty = true;
            return true;
        }

        public bool Modifica(Libro libro)
        {
            if (!Contains(libro) || libro != _libri[libro.Id])
                return false;
            _isDirty = true;
            return true;
        }

        //  --- Gestione persone ---

        public int GetNextIdPersona()
        {
            return _nextIdPersona++;
        }

        public Persona GetPersona(int id)
        {
            return _persone[id];
        }

        public IEnumerable<Persona> GetPersone()
        {
            return _persone.Values;
        }

        public bool Contains(Persona persona)
        {
            if (persona == null)
                throw new ArgumentNullException("persona");
            return _persone.ContainsKey(persona.Id);
        }

        public bool Aggiungi(Persona persona)
        {
            if (Contains(persona))
                return false;
            _persone.Add(persona.Id, persona);
            _isDirty = true;
            return true;
        }

        public bool Modifica(Persona persona)
        {
            if (!Contains(persona) || persona != _persone[persona.Id])
                return false;
            _isDirty = true;
            return true;
        }

        //  --- Gestione prestiti ---

        public Dictionary<int, Prestito> Prestiti
        {
            get { return _prestiti; }
        }

        public IEnumerable<Prestito> GetPrestiti()
        {
            return _prestiti.Values;
        }

        public bool IsLibroDisponibile(Libro libro)
        {
            foreach (Prestito prestito in _prestiti.Values)
            {
                if (prestito.Libro == libro)
                    return false;
            }
            return true;
        }

        public IEnumerable<Libro> GetLibriDisponibili()
        {
            return from libro in _libri.Values
                   where libro.Disponibile
                   select libro;
        }

        public bool Contains(Prestito prestito)
        {
            if (prestito == null)
                throw new ArgumentNullException("persona");
            return _prestiti.ContainsKey(prestito.IdLibro);
        }

        public bool Aggiungi(Prestito prestito)
        {
            if (Contains(prestito))
                return false;
            _prestiti.Add(prestito.IdLibro, prestito);
            _isDirty = true;
            return true;
        }

        public bool Rimuovi(Prestito prestito)
        {
            if (!Contains(prestito))
                return false;
            _prestiti.Remove(prestito.IdLibro);
            _isDirty = true;
            return true;
        }

        public override string ToString()
        {
            return "Biblioteca " + GetHashCode();
        }
    }
}
